/*
 * $Id: DocumentationPipeline.java 356 2011-07-03 13:54:56Z balder $
 *
 * This file is part of the iText (R) project.
 * Copyright (c) 1998-2011 1T3XT BVBA
 * Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3
 * as published by the Free Software Foundation with the addition of the
 * following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 1T3XT,
 * 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL:
 * http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License,
 * a covered work must retain the producer line in every PDF that is created
 * or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing
 * a commercial license. Buying such a license is mandatory as soon as you
 * develop commercial activities involving the iText software without
 * disclosing the source code of your own applications.
 * These activities include: offering paid services to customers as an ASP,
 * serving PDFs on the fly in a web application, shipping iText with a closed
 * source product.
 *
 * For more information, please contact iText Software Corp. at this
 * address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.pdf;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Image;
import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.itextpdf.text.pdf.PdfDestination;
import com.itextpdf.text.pdf.PdfOutline;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.NoCustomContextException;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.PipelineException;
import com.itextpdf.tool.xml.ProcessObject;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import com.itextpdf.tool.xml.css.StyleAttrCSSResolver;
import com.itextpdf.tool.xml.exceptions.CssResolverException;
import com.itextpdf.tool.xml.html.HeaderNode;
import com.itextpdf.tool.xml.pipeline.AbstractPipeline;
import com.itextpdf.tool.xml.pipeline.ctx.MapContext;
import com.itextpdf.tool.xml.pipeline.end.PdfWriterPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipelineContext;
import com.itextpdf.tool.xml.pipeline.html.ImageProvider;
import com.itextpdf.tool.xml.pipeline.html.LinkProvider;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
public class DocumentationPipeline extends AbstractPipeline<DocumentationPipelineContext> {

	private static final Logger LOG = LoggerFactory.getLogger(DocumentationPipeline.class);
	private final Config config;

	/**
	 * @param config the configuration.
	 * @param next the next pipeline
	 */
	public DocumentationPipeline(final Config config, final Pipeline<?> next) {
		super(next);
		this.config = config;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.itextpdf.tool.xml.pipeline.AbstractPipeline#init(com.itextpdf.tool
	 * .xml.WorkerContext)
	 */
	@Override
	public Pipeline<?> init(final WorkerContext context) throws PipelineException {
		DocumentationPipelineContext ctx = new DocumentationPipelineContext();
		ctx.setResolver(config.css());
		context.put(getContextKey(), ctx);
		return super.init(context);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.itextpdf.tool.xml.pipeline.AbstractPipeline#open(com.itextpdf.tool
	 * .xml.Tag, com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> open(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		try {
			if (checkValidPart(t)) {
				LOG.debug("Valid Part found");
				final DocumentationPipelineContext dpc = getLocalContext(ctx);
				dpc.partOpen();
				final Map<String, String> attr = t.getAttributes();
				if (dpc.isBookmarking() && !"false".equalsIgnoreCase(attr.get("bookmark"))) {
					String title = attr.get("title");
					int level = dpc.partLevel();
					HeaderNode tree = dpc.getOutline();
					int lastLevel = tree.level();
					if (lastLevel == level) {
						tree = tree.parent();
					} else if (lastLevel > level) {
						while (lastLevel >= level) {
							lastLevel = tree.parent().level();
							tree = tree.parent();
						}
					}
					PdfDestination destination = new PdfDestination(PdfDestination.XYZ, 20,
							dpc.getWriter().getVerticalPosition(false), 0);
					dpc.setOutline(new HeaderNode(level, new PdfOutline(tree.outline(), destination, title), tree));
				}
				String fName = attr.get("file");
				FileInputStream in = new FileInputStream(new File(config.docRoot(), fName));
				LOG.debug("reading from "+ fName);
				config.stream().switchStream(in);
			} else if ("css".equalsIgnoreCase(t.getName())) {
				final DocumentationPipelineContext dpc = getLocalContext(ctx);
				final Map<String, String> attr = t.getAttributes();
				if (attr.containsKey("default") && "true".equalsIgnoreCase(attr.get("default"))) {
					((StyleAttrCSSResolver) dpc.getResolver())
							.addCss(XMLWorkerHelper.getInstance().getDefaultCSS());
				}
				if (attr.containsKey("file") && (!attr.containsKey("rel") || "pdf".equalsIgnoreCase(attr.get("rel")))) {
					try {
						dpc.getResolver().addCssFile(
								config.docRoot().getAbsolutePath() + File.separator + attr.get("file"), true);
					} catch (CssResolverException e) {
						throw new PipelineException(e);
					}
				}
			} else if ("pdf".equals(t.getName())) {
				final Map<String, String> attr = t.getAttributes();
				final MapContext cc = (MapContext) ctx.get(PdfWriterPipeline.class.getName());
				LOG.debug("Creating PDF");
				String name = attr.get("filename");
				FileOutputStream out = new FileOutputStream(new File(config.resultdir(), name));
				Document d = new Document();
				final DocumentationPipelineContext dpc = getLocalContext(ctx);
				dpc.setDocument(d);
				cc.put(PdfWriterPipeline.DOCUMENT, d);
				PdfWriter writer = PdfWriter.getInstance(d, out);
				cc.put(PdfWriterPipeline.WRITER, writer);
				if (attr.containsKey("author")) {
					d.addAuthor(attr.get("author"));
				}
				dpc.setDocument(d);
				dpc.setWriter(writer);
				// } else if ("header".equalsIgnoreCase(t.getName()) ||
				// "footer".equalsIgnoreCase(t.getName())) {
				// final DocumentationPipelineContext localContext =
				// getLocalContext();
				// HtmlPipelineContext hpc = (HtmlPipelineContext)
				// getContext().get(HtmlPipeline.class);
				// final boolean isHeader =
				// "header".equalsIgnoreCase(t.getName());
				// ElementHandlerImplementation handler = new
				// ElementHandlerImplementation();
				// if(isHeader) {
				// localContext.setHeaderHandler(handler);
				// } else {
				// localContext.setFooterHandler(handler);
				// }
				// localContext.setNextPipeline(getNext());
				// HtmlPipeline p;
				// try {
				// p = new HtmlPipeline(hpc.clone(), new
				// ElementHandlerPipeline(handler , getNext()));
				// p.setContext(getContext());
				// setNext(p);
				// } catch (CloneNotSupportedException e) {
				// throw new PipelineException(e);
				// }
			} else if ("imagedir".equalsIgnoreCase(t.getName())) {
				final Map<String, String> attr = t.getAttributes();
				HtmlPipelineContext hpc = (HtmlPipelineContext) ctx.get(HtmlPipeline.class.getName());
				hpc.setImageProvider(new ImageProvider() {
					private final HashMap<String, Image> map = new HashMap<String, Image>();

					public void store(final String src, final Image img) {
						map.put(src, img);
					}

					public Image retrieve(final String src) {
						return map.get(src);
					}

					public void reset() {
						map.clear();
					}

					public String getImageRootPath() {
						return attr.get("dir");
					}
				});
			} else if ("rootlink".equalsIgnoreCase(t.getName())) {
				final Map<String, String> attr = t.getAttributes();
				HtmlPipelineContext hpc = (HtmlPipelineContext) ctx.get(HtmlPipeline.class.getName());
				hpc.setLinkProvider(new LinkProvider() {

					public String getLinkRoot() {
						return attr.get("url");
					}
				});
			} else if ("body".equalsIgnoreCase(t.getName())) {
				final Map<String, String> attr = t.getAttributes();
				String bm = attr.get("bookmarks");
				final DocumentationPipelineContext dpc = getLocalContext(ctx);
				dpc.getDocument().open();
				if ("true".equalsIgnoreCase(bm)) {
					dpc.isBookmarking(true);
					dpc.setOutline(new HeaderNode(0, dpc.getWriter().getRootOutline(), null));
				}
			}
		} catch (NoCustomContextException e) {
			throw new PipelineException(e);
		} catch (FileNotFoundException e) {
			throw new PipelineException(e);
		} catch (DocumentException e) {
			throw new PipelineException(e);
		} catch (UnsupportedEncodingException e) {
			throw new PipelineException(e);
		}
		return super.open(ctx, t, po);
	}

	private boolean checkValidPart(final Tag t) {
		return "part".equalsIgnoreCase(t.getName()) && (!t.getAttributes().containsKey("rel"))
				|| "pdf".equalsIgnoreCase(t.getAttributes().get("rel"));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.itextpdf.tool.xml.pipeline.AbstractPipeline#close(com.itextpdf.tool
	 * .xml.Tag, com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> close(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		if (checkValidPart(t)) {
			DocumentationPipelineContext dpc = getLocalContext(ctx);
			dpc.partClose();
			if (dpc.isBookmarking() && !"false".equalsIgnoreCase(t.getAttributes().get("bookmark"))) {
				dpc.setOutline(dpc.getOutline().parent());
			}

		} else if ("pdf".equals(t.getName())) {
			MapContext cc;
			try {
				cc = (MapContext) ctx.get(PdfWriterPipeline.class.getName());
				Document document = (Document) cc.get(PdfWriterPipeline.DOCUMENT);
				document.newPage();
				document.close();
			} catch (NoCustomContextException e) {
				throw new PipelineException(e);
			}
			LOG.info("Done Writing PDF");
		}
		// else if ("header".equalsIgnoreCase(t.getName()) ||
		// "footer".equalsIgnoreCase(t.getName())) {
		// final DocumentationPipelineContext localContext = getLocalContext();
		// if (null == localContext.getPageEvent()) {
		// PdfPageEventHelper helper = new PdfPageEventHelper() {
		// /* (non-Javadoc)
		// * @see
		// com.itextpdf.text.pdf.PdfPageEventHelper#onStartPage(com.itextpdf.text.pdf.PdfWriter,
		// com.itextpdf.text.Document)
		// */
		// @Override
		// public void onStartPage(final PdfWriter writer, final Document
		// document) {
		// if (null != localContext.getHeaderHandler()) {
		// for (Element e : localContext.getHeaderHandler().getElements()) {
		// try {
		// document.add(e);
		// } catch (DocumentException e1) {
		// e1.printStackTrace();
		// }
		// }
		// }
		// }
		//
		// /* (non-Javadoc)
		// * @see
		// com.itextpdf.text.pdf.PdfPageEventHelper#onEndPage(com.itextpdf.text.pdf.PdfWriter,
		// com.itextpdf.text.Document)
		// */
		// @Override
		// public void onEndPage(final PdfWriter writer, final Document
		// document) {
		// if (null != localContext.getFooterHandler()) {
		// for (Element e : localContext.getFooterHandler().getElements()) {
		// try {
		// document.add(e);
		// } catch (DocumentException e1) {
		// e1.printStackTrace();
		// }
		// }
		// }
		// }
		// };
		// localContext.setPageEvent(helper);
		// localContext.getWriter().setPageEvent(localContext.getPageEvent());
		// LOG.debug("Setting page event " + localContext.getPageEvent());
		// }
		//
		// setNext(localContext.getNextPipeline());
		// }
		return super.close(ctx, t, po);
	}

}
