/*
 * $Id: SitePipeline.java 356 2011-07-03 13:54:56Z balder $
 *
 * This file is part of the iText (R) project.
 * Copyright (c) 1998-2011 1T3XT BVBA
 * Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3
 * as published by the Free Software Foundation with the addition of the
 * following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 1T3XT,
 * 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL:
 * http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License,
 * a covered work must retain the producer line in every PDF that is created
 * or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing
 * a commercial license. Buying such a license is mandatory as soon as you
 * develop commercial activities involving the iText software without
 * disclosing the source code of your own applications.
 * These activities include: offering paid services to customers as an ASP,
 * serving PDFs on the fly in a web application, shipping iText with a closed
 * source product.
 *
 * For more information, please contact iText Software Corp. at this
 * address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.site;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import org.apache.commons.io.FileUtils;

import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.itextpdf.tool.xml.Experimental;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.PipelineException;
import com.itextpdf.tool.xml.ProcessObject;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.pipeline.AbstractPipeline;
import com.itextpdf.tool.xml.pipeline.ctx.MapContext;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
@Experimental
public class SitePipeline extends AbstractPipeline<MapContext> {

	private static final Logger LOG = LoggerFactory.getLogger(SitePipeline.class);
	private final File srcdir;
	private final File outdir;
	private final SiteWriter sitewriter;
	private final AttributeUtil attrUtil;

	/**
	 * @param srcdir the source directory
	 * @param outdir the output directory
	 * @param next the next pipeline in line
	 */
	public SitePipeline(final File srcdir, final File outdir, final Pipeline<?> next) {
		super(next);
		this.srcdir = srcdir;
		this.outdir = outdir;
		this.sitewriter = new SiteWriter();
		this.attrUtil = new AttributeUtil();
	}

	/**
	 * Create tree.js:
	 *
	 * <pre>
	 *  var tree = {
	 *  	menu : [ {name: "info", url : "info.html", id : ""}, {name: "links", url: "links.html", id :""}],
	 *  	title: "",
	 * 	map : []
	 * };
	 * </pre>
	 */
	@Override
	public Pipeline<?> open(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		try {
			if (checkValidPart(t)) {
				createPart(t, ctx);
			} else if (checkValidCss(t)) {
				createCSS(t, ctx);
			} else if ("body".equalsIgnoreCase(t.getName())) {
				createBody(ctx);
			} else if ("pdf".equals(t.getName())) {
				createPDF(t, ctx);
			} else if ("header".equalsIgnoreCase(t.getName()) && attrUtil.contains("site", t.getAttributes().get("rel"))) {
				createHeader(t, ctx);
			} else if ("footer".equalsIgnoreCase(t.getName()) && attrUtil.contains("site", t.getAttributes().get("rel"))) {
				createFooter(t, ctx);
			}
		} catch (IOException e) {
			throw new PipelineException("Unable to copy default site", e);
		}
		return super.open(ctx, t, po);
	}

	private void createFooter(final Tag t, final WorkerContext ctx) throws PipelineException {
		LOG.debug("itextdoc sitebuilder detected footer file " + t.getAttributes().get("file"));
		final MapContext mapContext = getLocalContext(ctx);
		mapContext.put("footer", t.getAttributes().get("file"));
	}

	private void createHeader(final Tag t, final WorkerContext ctx) throws PipelineException, FileNotFoundException {
		LOG.debug("itextdoc sitebuilder detected header file " + t.getAttributes().get("file"));
		final MapContext mapContext = getLocalContext(ctx);
		mapContext.put("header", new FileInputStream(new
					File(srcdir, t.getAttributes().get("file"))));
	}

	private void createPDF(final Tag t, final WorkerContext ctx) throws PipelineException, FileNotFoundException, IOException {
		final MapContext mapContext = getLocalContext(ctx);
		final FileOutputStream o = new FileOutputStream(new File(outdir, "index.html"));
		mapContext.put("out", o);
		sitewriter.writeHeadOpen(o, System.getProperty("file.encoding"));
		String title = t.getAttributes().get("title");
		if (null == title) {
			title = "Generated Site by itextdoc-plugin";
		}
		sitewriter.writeTitle(o, title);
		mapContext.put("title", title);
		sitewriter.writeScript(o, "jquery.js");
		sitewriter.writeScript(o, "tree.js");
		final URL jquery = SitePipeline.class.getResource("/site/jquery.js");
		FileUtils.copyURLToFile(jquery, new File(outdir, "jquery.js"));
		final StringBuilder b = (StringBuilder) mapContext.get("buffer");
		b.append("var tree = {");
		if (t.getAttributes().containsKey("title")) {
			LOG.debug("Creating ajax site " + title);
			b.append(" title: \"").append(title).append("\",");
		}
		b.append(" menu: {");
	}

	private void createBody(final WorkerContext ctx) throws PipelineException, IOException {
		final MapContext mapContext = getLocalContext(ctx);
		FileOutputStream o = (FileOutputStream) mapContext.get("out");
		if ("false".equals(mapContext.get("gotcss"))) {
			LOG.warn("itextdoc sitebuilder No CSS Detected for Site, using default ");
			writeDefaultCSS(o);
		}
		sitewriter.writeHeadCloseBodyOpen(o);
		sitewriter.write(o, "<div id=\"title\">").write(o, (String) mapContext.get("title")).write(o, "</div>");
		Object object = mapContext.get("header");
		if (null != object) {
			sitewriter.writeDivContent(o, (InputStream) object, "header");
		} else {
			sitewriter.write(o, "<div id=\"header\"></div>");
		}
		sitewriter.writeContent(o, SitePipeline.class.getResourceAsStream("/site/site.html"));
	}

	private void createCSS(final Tag t, final WorkerContext ctx) throws PipelineException, IOException {
		final Map<String, String> attr = t.getAttributes();
		final MapContext mapContext = getLocalContext(ctx);
		if (attr.containsKey("file")) {
			final String fileStr = attr.get("file");
			LOG.debug("itextdoc sitebuilder detected css file " + fileStr);
			final FileOutputStream o = (FileOutputStream) mapContext.get("out");
			if (attr.containsKey("default") && "true".equalsIgnoreCase(attr.get("default"))) {
				writeDefaultCSS(o);
			}
			mapContext.put("gotcss", "true");
			FileUtils.copyFile(new File(srcdir, fileStr), new File(outdir, fileStr));
			sitewriter.writeStyleLink(o, fileStr);
		}
	}

	private void createPart(final Tag t, final WorkerContext ctx) throws PipelineException {
		String title = t.getAttributes().get("title");
		final MapContext mapContext = getLocalContext(ctx);
		final StringBuilder b = (StringBuilder) mapContext.get("buffer");
		String file = t.getAttributes().get("file");
		Integer cnt = (Integer) mapContext.get("counter");
		cnt++;
		mapContext.put("counter", cnt);
		try {
			FileUtils.copyFile(new File(srcdir, file), new File(outdir, file));
		} catch (IOException e) {
			throw new PipelineException("Could not copy file", e);
		}
		StringBuilder titleb = new StringBuilder();
		Tag local = t;
		while ("part".equals(local.getParent().getName())) {
			titleb.append("&gt; ");
			local = local.getParent();
		}
		titleb.append(title);
		b.append(" \"itextdoc-menu-" + cnt.toString() + "\" : ").append("{ name: \"").append(titleb)
				.append("\", id: \"itextdoc-menu-" + cnt.toString() + "\", url: \"").append(file)
				.append("\" } ,");
	}

	/**
	 * @param t check if this tag is a valid css tag for site
	 * @return true if css is found and attribute rel="site"
	 */
	public boolean checkValidCss(final Tag t) {
		return ("css".equalsIgnoreCase(t.getName()) && attrUtil.contains("site", t.getAttributes().get("rel")));
	}

	/**
	 * @param t check if this tag is a valid tag for site
	 * @return true if part is found and if there is a attribute rel that it contains 'site'
	 */
	public boolean checkValidPart(final Tag t) {
		return ("part".equalsIgnoreCase(t.getName()) && (!t.getAttributes().containsKey("rel") || attrUtil.contains("site", t.getAttributes().get("rel"))));
	}

	/**
	 * @param o the OutputStream to write to.
	 * @throws IOException if css could not be written
	 */
	protected void writeDefaultCSS(final OutputStream o) throws IOException {
		sitewriter.writeStyleLink(o, "default.css");
		URL css = SitePipeline.class.getResource("/site/style.css");
		FileUtils.copyURLToFile(css, new File(outdir, "default.css"));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.itextpdf.tool.xml.pipeline.A,;:
	 * ujchhcjhjkjjhh-<bstractPipeline#close(com.itextpdf.tool .xml.Tag,
	 * com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> close(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		if ("pdf".equalsIgnoreCase(t.getName())) {
			closePDF(ctx);
		} else if ("cp".equalsIgnoreCase(t.getName())) {

			try {
				File srcFile = new File(srcdir, t.getAttributes().get("rs"));
				File destFile =  new File(outdir, t.getAttributes().get("rs"));
				if (srcFile.isDirectory()) {
					LOG.debug("Copying Resource Directory " + srcFile.getPath());
					FileUtils.copyDirectory(srcFile, destFile);
				} else {
					LOG.debug("Copying Resource File " + srcFile.getPath());
					FileUtils.copyFile(srcFile, destFile);
				}
			} catch (IOException e) {
				throw new PipelineException(e);
			}
		}
		return super.close(ctx, t, po);
	}

	private void closePDF(final WorkerContext ctx) throws PipelineException {
		final MapContext mapContext = getLocalContext(ctx);
		final StringBuilder b = (StringBuilder) mapContext.get("buffer");
		final int index = b.length() - 1;
		if (',' == b.charAt(index)) {
			b.deleteCharAt(index);
		}
		final SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HH:mm");
		b.append("}, buildon: \"" + sdf.format(new Date()) + "\"};");
		FileOutputStream out = null;
		FileOutputStream o = null;
		try {
			LOG.info("Writing ajax site - index.html");
			out = new FileOutputStream(new File(outdir, "tree.js"));
			out.write(b.toString().getBytes());
			o = (FileOutputStream) mapContext.get("out");
			final Object object = mapContext.get("footer");
			if (null != object) {
				sitewriter.writeDivContent(o, new FileInputStream(new File(srcdir, (String) object)), "footer");
			} else {
				sitewriter.write(o, "<div id=\"footer\"></div>");
			}
			sitewriter.writeContent(o, SitePipeline.class.getResourceAsStream("/site/notice.html"));
			sitewriter.writeBodyClose(o);
		} catch (final FileNotFoundException e) {
			throw new PipelineException("Unable to create tree.js", e);
		} catch (final IOException e) {
			throw new PipelineException("Unable to write tree.js", e);
		} finally {
			try {
				out.close();
			} catch (final IOException e) {
			} finally {
				try {
					o.close();
				} catch (final IOException e) {
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see com.itextpdf.tool.xml.pipeline.AbstractPipeline#init(com.itextpdf.tool.xml.WorkerContext)
	 */
	@Override
	public Pipeline<?> init(final WorkerContext context) throws PipelineException {
		MapContext mapContext = new MapContext();
		mapContext.put("buffer", new StringBuilder());
		mapContext.put("counter", Integer.valueOf(0));
		mapContext.put("gotcss", "false");
		context.put(getContextKey(), mapContext);
		return super.init(context);
	}
}
